
/*
***************************************************************************
Copyright (C) 2000-2010 KBasic Software: www.kbasic.com. All rights reserved.
Source codes are copyrighted by Bernd Noetscher 2000-2010.

You may use this source codes under the terms of the GNU Public License (Version 3) as published 
by the Free Software Foundation. 

If you are interested in using the source codes for other licenses or 
commercial development, you must buy commercial licenses from KBasic Software.
***************************************************************************
*/


#ifndef _Q7BSERIALPORT_H
#define _Q7BSERIALPORT_H

#include <QtCore>
#include "qextserialport.h"



class Q7BSerialPort : public QObject
{
  Q_OBJECT
public:

//Q7BSerialPort();


Q7BSerialPort(QObject *parent = 0) : QObject (parent)
{  

}

~Q7BSerialPort()
{
  t.stop();
  if (sp.isOpen()) sp.close();
}

bool Open()
{    
  if (sp.open(QIODevice::ReadWrite | QIODevice::Unbuffered)){
 
    SetTimeOutMillisec(500);
    SetPollingInterval(500);

    connect(&t, SIGNAL(timeout()), this, SLOT(OnEvent()));

    nBytesRead = 0;
    nBytesWritten = 0;

    SetPortName(PortName());

    SetBaudRate(BaudRate());

    SetDataBits(DataBits());

    SetFlowControl(FlowControl());

    SetParity(Parity());

    SetStopBits(StopBits());

   // SetTimeOutMillisec(TimeOutMillisec());

  //  SetPollingInterval(PollingInterval());

    t.start();
  }
  return sp.isOpen();
}

qint64 nBytesRead;
qint64 nBytesWritten;

qint64 BytesRead()
{
  return nBytesRead;
}

qint64 BytesWritten()
{
  return nBytesWritten;
}


QString ReadLine()
{   
  static char ac[4096];
  nBytesRead = sp.readLine(&ac[0], 4096);
  if (nBytesRead > 0) return QString::fromAscii(ac);

  return "";  
}

void WriteLine(const QString & s)
{  
  QByteArray a = s.toAscii();  
  nBytesWritten = sp.write(a.data(), a.length());  
}

QString ReadData()
{
  return ReadLine();
}

void WriteData(const QString & s)
{
  WriteLine(s);
}

bool IsOpen()
{
  return sp.isOpen();
}

bool IsReadable()
{
  return sp.isReadable();
}

bool IsWritable()
{
  return sp.isWritable();
}

void Close()
{
  if (sp.isOpen()){
    t.stop();
    sp.close();
  }
}

void Flush()
{
  if (sp.isOpen()) sp.flush();
}

qint64 BytesAvailable()
{
  return sp.bytesAvailable();
}

void SetDtr(bool b)
{
  sp.setDtr(b);
}

void SetRts(bool b)
{
  sp.setRts(b);
}

bool AtEnd()
{
  return sp.atEnd();
}

QString LineStatus()
{
  ulong n = sp.lineStatus();
  QString s = "";
  if (n & LS_CTS) if (!s.isEmpty()) s += ","; s += "CTS";
  if (n & LS_DSR) if (!s.isEmpty()) s += ","; s += "DSR";
  if (n & LS_DCD) if (!s.isEmpty()) s += ","; s += "DCD";
  if (n & LS_RI) if (!s.isEmpty()) s += ","; s += "RI";
  return s;
}

bool IsCTS()
{
  ulong n = sp.lineStatus();
  return n & LS_CTS;
}

bool IsDSR()
{
  ulong n = sp.lineStatus();
  return n & LS_DSR;
}

bool IsDCD()
{
  ulong n = sp.lineStatus();
  return n & LS_DCD;
}

bool IsRI()
{
  ulong n = sp.lineStatus();
  return n & LS_RI;
}

QString LastError()
{
  ulong l = sp.lastError();

  QString s = "";

  if (l == E_NO_ERROR) s = "NO_ERROR";
  else if (l == E_INVALID_FD) s = "INVALID_FD";
  else if (l == E_NO_MEMORY) s = "NO_MEMORY";
  else if (l == E_CAUGHT_NON_BLOCKED_SIGNAL) s = "CAUGHT_NON_BLOCKED_SIGNAL";
  else if (l == E_PORT_TIMEOUT) s = "PORT_TIMEOUT";
  else if (l == E_INVALID_DEVICE) s = "INVALID_DEVICE";
  else if (l == E_BREAK_CONDITION) s = "BREAK_CONDITION";
  else if (l == E_FRAMING_ERROR) s = "FRAMING_ERROR";
  else if (l == E_IO_ERROR) s = "IO_ERROR";
  else if (l == E_BUFFER_OVERRUN) s = "BUFFER_OVERRUN";
  else if (l == E_RECEIVE_OVERFLOW) s = "RECEIVE_OVERFLOW";
  else if (l == E_RECEIVE_PARITY_ERROR) s = "RECEIVE_PARITY_ERROR";
  else if (l == E_TRANSMIT_OVERFLOW) s = "TRANSMIT_OVERFLOW";
  else if (l == E_READ_FAILED) s = "READ_FAILED";
  else if (l == E_WRITE_FAILED) s = "WRITE_FAILED";
 
  return s;
}

QString LastErrorText()
{
  ulong l = sp.lastError();

  QString s = "";

  if (l == E_NO_ERROR) s = "No Error has occured.";
  else if (l == E_INVALID_FD) s = "Invalid file descriptor (port was not opened correctly).";
  else if (l == E_NO_MEMORY) s = "Unable to allocate memory tables (POSIX).";
  else if (l == E_CAUGHT_NON_BLOCKED_SIGNAL) s = "Caught a non-blocked signal (POSIX).";
  else if (l == E_PORT_TIMEOUT) s = "Operation timed out (POSIX).";
  else if (l == E_INVALID_DEVICE) s = "The file opened by the port is not a character device (POSIX).";
  else if (l == E_BREAK_CONDITION) s = "The port detected a break condition.";
  else if (l == E_FRAMING_ERROR) s = "The port detected a framing error.";
  else if (l == E_IO_ERROR) s = "There was an I/O error while communicating with the port (usually caused by incorrect baud rate settings).";
  else if (l == E_BUFFER_OVERRUN) s = "Character buffer overrun.";
  else if (l == E_RECEIVE_OVERFLOW) s = "Receive buffer overflow.";
  else if (l == E_RECEIVE_PARITY_ERROR) s = "The port detected a parity error in the received data.";
  else if (l == E_TRANSMIT_OVERFLOW) s = "Transmit buffer overflow.";
  else if (l == E_READ_FAILED) s = "General read operation failure.";
  else if (l == E_WRITE_FAILED) s = "General write operation failure.";
 
  return s;
}

QString PortName()
{
  return sp.portName();
}

void SetPortName(const QString & n)
{ 
  sp.setPortName(n);
}

QString BaudRate()
{
  BaudRateType n = sp.baudRate();

  QString s = "";
  if (n == BAUD50) s = "BAUD50";
  else if (n == BAUD75) s = "BAUD75";
  else if (n == BAUD110) s = "BAUD110";
  else if (n == BAUD134) s = "BAUD134";
  else if (n == BAUD150) s = "BAUD150";
  else if (n == BAUD200) s = "BAUD200";
  else if (n == BAUD300) s = "BAUD300";
  else if (n == BAUD600) s = "BAUD600";
  else if (n == BAUD1200) s = "BAUD1200";
  else if (n == BAUD1800) s = "BAUD1800";
  else if (n == BAUD2400) s = "BAUD2400";
  else if (n == BAUD4800) s = "BAUD4800";
  else if (n == BAUD9600) s = "BAUD9600";
  else if (n == BAUD14400) s = "BAUD14400";
  else if (n == BAUD19200) s = "BAUD19200";
  else if (n == BAUD38400) s = "BAUD38400";
  else if (n == BAUD56000) s = "BAUD56000";
  else if (n == BAUD76800) s = "BAUD76800";
  else if (n == BAUD115200) s = "BAUD115200";
  else if (n == BAUD128000) s = "BAUD128000";
  else if (n == BAUD256000) s = "BAUD256000";

  return s;
}

void SetBaudRate(const QString & n)
{ 
  if (n.compare("BAUD50", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD50);
  else if (n.compare("BAUD75", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD75);
  else if (n.compare("BAUD110", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD110);
  else if (n.compare("BAUD134", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD134);
  else if (n.compare("BAUD150", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD150);
  else if (n.compare("BAUD200", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD200);
  else if (n.compare("BAUD300", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD300);
  else if (n.compare("BAUD600", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD600);
  else if (n.compare("BAUD1200", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD1200);
  else if (n.compare("BAUD1800", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD1800);
  else if (n.compare("BAUD2400", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD2400);
  else if (n.compare("BAUD4800", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD4800);
  else if (n.compare("BAUD9600", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD9600);
  else if (n.compare("BAUD14400", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD14400);
  else if (n.compare("BAUD19200", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD19200);
  else if (n.compare("BAUD38400", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD38400);
  else if (n.compare("BAUD56000", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD56000);
  else if (n.compare("BAUD76800", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD76800);
  else if (n.compare("BAUD115200", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD115200);
  else if (n.compare("BAUD128000", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD128000);
  else if (n.compare("BAUD256000", Qt::CaseInsensitive) == 0) sp.setBaudRate(BAUD256000);    
}

QString DataBits()
{
  DataBitsType n = sp.dataBits();

  QString s = "";
  if (n == DATA_5) s = "DATA_5";
  else if (n == DATA_6) s = "DATA_6";
  else if (n == DATA_7) s = "DATA_7";
  else if (n == DATA_8) s = "DATA_8";

  return s;
}

void SetDataBits(const QString & n)
{ 
  if (n.compare("DATA_5", Qt::CaseInsensitive) == 0) sp.setDataBits(DATA_5);
  else if (n.compare("DATA_6", Qt::CaseInsensitive) == 0) sp.setDataBits(DATA_6);
  else if (n.compare("DATA_7", Qt::CaseInsensitive) == 0) sp.setDataBits(DATA_7);
  else if (n.compare("DATA_8", Qt::CaseInsensitive) == 0) sp.setDataBits(DATA_8);
}

QString FlowControl()
{  
  FlowType n = sp.flowControl();

  QString s = "";
  if (n == FLOW_OFF) s = "FLOW_OFF";
  else if (n == FLOW_HARDWARE ) s = "FLOW_HARDWARE";
  else if (n == FLOW_XONXOFF ) s = "FLOW_XONXOFF";

  return s;  
}

void SetFlowControl(const QString & n)
{ 
  if (n.compare("FLOW_OFF", Qt::CaseInsensitive) == 0) sp.setFlowControl(FLOW_OFF);
  else if (n.compare("FLOW_HARDWARE", Qt::CaseInsensitive) == 0) sp.setFlowControl(FLOW_HARDWARE);
  else if (n.compare("FLOW_XONXOFF", Qt::CaseInsensitive) == 0) sp.setFlowControl(FLOW_XONXOFF);
}

QString Parity()
{
  ParityType n = sp.parity();

  QString s = "";
  if (n == PAR_NONE) s = "PAR_NONE";
  else if (n == PAR_ODD) s = "PAR_ODD";
  else if (n == PAR_EVEN) s = "PAR_EVEN";
  else if (n == PAR_MARK) s = "PAR_MARK";
  else if (n == PAR_SPACE) s = "PAR_SPACE";

  return s;  
}

void SetParity(const QString & n)
{ 
  if (n.compare("PAR_NONE", Qt::CaseInsensitive) == 0) sp.setParity(PAR_NONE);
  else if (n.compare("PAR_ODD", Qt::CaseInsensitive) == 0) sp.setParity(PAR_ODD);
  else if (n.compare("PAR_EVEN", Qt::CaseInsensitive) == 0) sp.setParity(PAR_EVEN);
  else if (n.compare("PAR_MARK", Qt::CaseInsensitive) == 0) sp.setParity(PAR_MARK);
  else if (n.compare("PAR_SPACE", Qt::CaseInsensitive) == 0) sp.setParity(PAR_SPACE);
}

QString StopBits()
{
  StopBitsType n = sp.stopBits();

  QString s = "";
  if (n == STOP_1) s = "STOP_1";
  else if (n == STOP_1_5) s = "STOP_1_5";
  else if (n == STOP_2) s = "STOP_2";

  return s;  
}

void SetStopBits(const QString & n)
{ 
  if (n.compare("STOP_1", Qt::CaseInsensitive) == 0) sp.setStopBits(STOP_1);
  else if (n.compare("STOP_1_5", Qt::CaseInsensitive) == 0) sp.setStopBits(STOP_1_5);
  else if (n.compare("STOP_2", Qt::CaseInsensitive) == 0) sp.setStopBits(STOP_2);
}

void SetTimeOutMillisec(qint64 n)
{ 
  sp.setTimeout(n); 
}

void SetPollingInterval(qint64 n)
{ 
  t.setInterval(n);  
}

QTimer t;

QextSerialPort sp;


public slots:
void OnEvent()
{
  emit Event();
}

signals:
void Event();

};

#endif