﻿/*
'***************************************************************************
'Copyright (C) 2010-2011 KBasic Software: www.kbasic.com. All rights reserved.
'Source codes are created by Bernd Noetscher sales@kbasic.com 
'
'You may use this source codes under the terms of the GNU Public License (Version 3) as published 
'by the Free Software Foundation. 
'
'If you are interested in using the source codes for other licenses or 
'commercial development, you must buy commercial licenses from KBasic Software.
'***************************************************************************
*/



#include "Q7BCompiler.h"


bool Parser::projectContainsClass(QString s)
{
    for (int i = 0; Me.bError == false && i < Me.files.size(); i++){
      if (Me.files.at(i) == s) return true;
    }
    return false;
}

void Parser::error(QString ss, bool bDetailedInfo)
{
  QString s = ss;
  if (Me.bError) return;

  QString token_2 = getToken(-3);
  QString token_1 = getToken(-2);
  QString token0 = getToken(-1);
  QString token1 = getToken();
  QString token2 = getToken(+1);

  // avoid caching problem when source code contains errors
  if (!Me.sFileName.isEmpty()) QFile::remove(CONFIG["PROJECTPATH"] + "/" + CONFIG["BUILDDIR"] + "/Scanner/" + Me.sFileName);

  //if (s.isEmpty()) 
  s = "syntax error or parser error " + (s.isEmpty() ? "" : (" - " + s));
#ifdef Q7BASIC
  s = "Q7Basic: " + s;
#elif defined OBJB
  s = "Objective-Basic: " + s;
#elif defined DROIDBASIC
  s = "DroidBasic: " + s;
#endif // Q7BASIC_OBJB

  if (token_1.mid(1) == "."){
    s += " " + QString("near '" + format(token_2) + " " + format(token_1) + " " + format(token0) + " " + format(token1) + " " + format(token2) + "'").simplified();
  } else {
    QString fsd = format(token0) + format(token1) + format(token2);
    if (!fsd.simplified().isEmpty()) s += " near '" + fsd + "'";
  }
  if (bDetailedInfo && !Me.sFileName.isEmpty()){
    s += QString(" in line %1 in file %2\n").arg(Me.nLine).arg(Me.sFileName);
  }
/*
  int nCount = 0;
  QString qwert = Me.sSourceCode[Me.sFileName].mid(Me.nTokenPos);
  for (int i = 0; i < qwert.length(); i++){
    if (qwert.at(i) == '\n'){
      nCount++;
    }
    if (nCount == 5){
      qwert = qwert.left(i);
      break;
    }
  }

  s += ("..." + qwert);*/

//  Parser::SaveFile("C:\\Users\\Bernd\\Desktop\\debug.txt", s, true);


  Write_To_stderr(s);

  Me.bError = true;
}

void Parser::CopyOverwrite(QString s, QString s2)
{
    if (QFile::exists(s2)) QFile::remove(s2);
    QFile::copy(s, s2);
}

QString Parser::format(QString s)
{
  if (s.left(1) == "S"){
     return "\"" + s.mid(1).replace("\n", " ") + "\"";
  }
  if (s.left(1) == "C"){
     return " '" + s.mid(1).replace("\n", " ");
  }
  return s.mid(1).replace("\n", " ");
}

void Parser::Write_To_stdout(const QString & s)
{
  QFile file;
  if (file.open(stdout, QIODevice::WriteOnly)){
    file.write(s.toLocal8Bit());
    file.flush();
  }
}

void Parser::Write_To_stderr(const QString & s)
{
  QFile file;
  if (file.open(stderr, QIODevice::WriteOnly)){    
    file.write(s.toLocal8Bit());
    file.flush();
  }
}

QString Parser::getPath(const QString & s)
{
  int n = s.lastIndexOf("/");
  QString sName = s;
  if (n == -1) n = s.lastIndexOf("\\");
  if (n != -1) sName = s.left(n + 1);
  return sName;
}

QString Parser::getFileName(const QString & s)
{
  int n = s.lastIndexOf("/");
  QString sName = s;
  if (n == -1) n = s.lastIndexOf("\\");
  if (n != -1) sName = s.mid(n + 1);
  return sName;
}

QString Parser::getBaseName(const QString & ss)
{
  QString s = ss;
  int n = s.lastIndexOf("/");
  if (n == -1) n = s.lastIndexOf("\\");
  if (n != -1) s = s.mid(n + 1);
  n = s.lastIndexOf(".");
  if (n != -1) s = s.left(n);
  return s;
}

QString Parser::getExtension(const QString & ss)
{
  QString s = ss;
  int n = s.lastIndexOf(".");  
  if (n != -1) s = s.mid(n + 1);
  return s;
}

QString Parser::getSignalOutletName(const QString & ss)
{
  QString s;

#ifdef Q7BASIC
  if (ss.startsWith("on_")){
    int n = ss.lastIndexOf("_");
    if (n >= 0){
      s = ss.mid(3, n - 3);
    }
  }
#elif defined OBJB  
#elif defined DROIDBASIC
  {
    int n = ss.lastIndexOf("_");
    if (n >= 0){
      s = ss.mid(0, n);
    }
  }
#endif // Q7BASIC_OBJB


  return s;
}

QString Parser::getSignalSignalName(const QString & ss)
{
  QString s;


#ifdef Q7BASIC
  if (ss.count("_") < 2) return "";

  int n = ss.lastIndexOf("_");
  if (n >= 0){
    s = ss.mid(n + 1);
  }
#elif defined OBJB  
#elif defined DROIDBASIC
  if (ss.count("_") < 1) return "";

  int n = ss.lastIndexOf("_");
  if (n >= 0){
    s = ss.mid(n + 1);
  }
#endif // Q7BASIC_OBJB

  
  return s;
}

QString Parser::getFileNameForClass(const QString & s)
{
  return Me.sFileNameForClass[s];
}

QString Parser::getClassName(const QString & ss)
{
  if (CONFIG["SINGLEFILEPROJECT"] == "true"){
    return "Global";
  }

  QString s = ss;
  int n = s.lastIndexOf(".");  
  if (n != -1){
    s = s.left(n);
  }  
  n = s.lastIndexOf(".");  
  if (n != -1){
    s = s.left(n);
  }

  return s;
}

QString Parser::getSuperClassName(const QString & ss)
{
#ifdef Q7BASIC
  if (CONFIG["SINGLEFILEPROJECT"] == "true"){
    return "QObject";
  }

#elif defined(OBJB)
#endif // Q7BASIC_OBJB

  QString s = ss;
  int n = s.lastIndexOf(".");  
  if (n != -1){
    s = s.left(n);
  }  
  n = s.lastIndexOf(".");  
  if (n != -1){
    s = s.mid(n + 1);
  }

#ifdef Q7BASIC
#elif defined(OBJB)

  if (s == "Array") s = "NSMutableArray";
  else if (s == "Dictionary") s = "NSMutableDictionary";
  else if (s == "Data") s = "NSMutableData";

#endif // Q7BASIC_OBJB
  return s;


}

QString Parser::getSuperClassNameOfType(const QString & sClass)
{
  if (Me.classes.contains(sClass)) return Me.classes[sClass];
  if (Me.controls.contains(sClass)) return Me.controls[sClass];
  if (Me.objects.contains(sClass)) return Me.objects[sClass];

  if (Me.Classes.contains(sClass)) return Me.Classes[sClass];

  return "";
}

QString Parser::LoadFile(const QString & sFile)
{
  QString s;
  QString sFile2=sFile;
#ifndef Q_OS_WIN32
  sFile2 = sFile2.replace("\\", "/");
#endif
 QFile f(sFile2);
  if (!f.open(QIODevice::ReadOnly)){
    error(QString("Could not read from %1").arg(sFile));
  } else {
    QTextStream t(&f);
    
    t.setCodec("UTF-8");
    t.setAutoDetectUnicode(true);
    t.setGenerateByteOrderMark(true);

    s = t.readAll();
    s = s.replace("\r", "");
    f.close();
  }
  return s;
}

QStringList Parser::LoadFileAsStringList(const QString & sFile)
{
  QStringList sl;
  QString sFile2=sFile;
#ifndef Q_OS_WIN32
  sFile2 = sFile2.replace("\\", "/");
#endif
 QFile f(sFile2);
  if (!f.open(QIODevice::ReadOnly)){
    error(QString("Could not read from %1").arg(sFile));
  } else {
    QDataStream t(&f);

    t >> sl;
    f.close();
  }
  return sl;
}

void Parser::SaveFile(const QString & sFile, const QString & s, bool bUTF8)
{

   QString sFile2=sFile;
#ifndef Q_OS_WIN32
   sFile2 = sFile2.replace("\\", "/");
#endif
  QFile f(sFile2);
  if (!f.open(QIODevice::WriteOnly)){//C:\Users\Bernd\Desktop\objb_q7basic\q7b\Runtime
    error(QString("Could not write to %1").arg(sFile));
  } else {
    QTextStream t(&f);
    
    if (bUTF8){
      t.setCodec("UTF-8");
      t.setAutoDetectUnicode(true);
      t.setGenerateByteOrderMark(true);
    }

    t << s;
    f.close();    
  }
}

void Parser::SaveFileAsStringList(const QString & sFile, const QStringList & sl)
{
    QString sFile2=sFile;
 #ifndef Q_OS_WIN32
    sFile2 = sFile2.replace("\\", "/");
 #endif
   QFile f(sFile2);
  if (!f.open(QIODevice::WriteOnly)){
    error(QString("Could not write to %1").arg(sFile));
  } else {
    QDataStream t(&f);

    t << sl;
    f.close();
    
  }
}

//QMap<QString, bool> writeAlwaysDueToLink;

bool Parser::shellCopy(QString a, QString b)
  {
    return false;
//return QProcess::execute("cp", QString("-r " + a + " " + b).split(" "));
 }


QMap<QString, QString> links;
bool Parser::copyDir(const QString& from, const QString& to, bool bCopyFilesEvenExists, QStringList ignoreFilter, bool bFirst, QString ignoreFilterEndsWith, QString ignoreFilterEndsWith2)
{
  if (bFirst){
      links.clear();
  }

  QDir d;
  d.mkpath(to);

 // Write_To_stdout(to + "\n");

  QDir thisDir( from );	
  QFileInfoList files = thisDir.entryInfoList();

  if ( files.size() ) {

    QFileInfo fi;
    for (int i = 0; i < files.size(); ++i) {
      fi = files.at(i);

      QString ss = fi.absoluteFilePath();

///       Write_To_stdout(ss + "\n");

		  if (fi.fileName() == "." || fi.fileName() == ".."){
		      ; // nothing
                  } else if ( fi.isSymLink()){

                      if (ignoreFilter.contains(fi.fileName())) continue;
                      if (ignoreFilter.contains("." + fi.suffix())) continue;
                      
                      if (!ignoreFilterEndsWith.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith)) continue;
                      if (!ignoreFilterEndsWith2.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith2)) continue;

                      QString sLink = to + fi.absoluteFilePath().mid(from.length());
                      QString sTarget = fi.symLinkTarget();

                      sTarget = to + sTarget.mid(from.length());

                      links[sTarget] = sLink;
/*
                      QString sLink = to + fi.absoluteFilePath().mid(from.length());

                     if (bCopyFilesEvenExists == true || !QFile::exists(sLink)){
                         QString sTarget = fi.symLinkTarget();

                         sTarget = to + sTarget.mid(from.length());

                         if (!QFile::exists(sTarget)){
                             if (QFileInfo(fi.symLinkTarget()).isDir()){
                                 if (!copyDir(fi.symLinkTarget(), sLink, bCopyFilesEvenExists, ignoreFilter, false)) return false;

                             } else {

                                 copyIfNeeded(fi.symLinkTarget(), sLink);

                             }
                          }


                      }*/

                      /*
                      if (ignoreFilter.contains(fi.fileName())) continue;
                      if (ignoreFilter.contains("." + fi.suffix())) continue;

                      QString sLink = to + fi.absoluteFilePath().mid(from.length());

                     if (bCopyFilesEvenExists == true || !QFile::exists(sLink)){
                         QString sTarget = fi.symLinkTarget();

                         sTarget = to + sTarget.mid(from.length());

                         if (!QFile::exists(sTarget)){
                             if (QFileInfo(fi.symLinkTarget()).isDir()){
                                 QDir().mkpath(sTarget);
                             } else {
                                 QDir().mkpath(getPath(sTarget));
                                 SaveFile(sTarget, "", false);

                                 writeAlwaysDueToLink[sTarget] = true;
                             }
                          }

                         QFile::link(sTarget, sLink);
                      }
                      */
		  } else if ( fi.isDir() ){

                      QString asf = fi.fileName();
          if (ignoreFilter.contains(fi.fileName())) continue;
          if (ignoreFilter.contains("." + fi.suffix())) continue;

          if (!ignoreFilterEndsWith.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith)) continue;
          if (!ignoreFilterEndsWith2.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith2)) continue;
/*
          if (asf == "build"){
              asf = asf;
              QString safd = ignoreFilter.join(";");
                      safd = safd;
          }
*/
        if (!copyDir(fi.absoluteFilePath(), to + "/" + fi.fileName(), bCopyFilesEvenExists, ignoreFilter, false, ignoreFilterEndsWith, ignoreFilterEndsWith2)) return false;

      } else {

        if (ignoreFilter.contains(fi.fileName())) continue;
        if (ignoreFilter.contains("." + fi.suffix())) continue;

        if (!ignoreFilterEndsWith.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith)) continue;
        if (!ignoreFilterEndsWith2.isEmpty() && fi.fileName().endsWith(ignoreFilterEndsWith2)) continue;

        if (bCopyFilesEvenExists == true || (!QFile::exists(to + "/" + fi.fileName())/* || writeAlwaysDueToLink.contains(to + "/" + fi.fileName())*/)){
          copyIfNeeded(fi.absoluteFilePath(), to + "/" + fi.fileName());
        }

      }		    
	}
  
  }    

  if (bFirst){ // create symblinks after all other files and directories have been created
      QMapIterator<QString, QString> i(links);
      while (i.hasNext()) {
          i.next();

          QFile::link(i.key(), i.value());
      }
  }
  return true;
}

// copy resource folder    
void Parser::copyResourceFolder(QString & sResourceFiles, QString sPath, QString sPathTo, QString sPrefix)
{
  if (QFile::exists(sPath)){
    QDir("/").mkpath(sPathTo);

    QDir dir(sPath);
    QStringList ll = dir.entryList();

    for (int i = 0; i < ll.size(); i++){       
      if (QFileInfo(sPath + "/" + ll.at(i)).isDir()){
        if (ll.at(i) != "." && ll.at(i) != ".."){
          copyResourceFolder(sResourceFiles, sPath + "/" + ll.at(i), sPathTo + "/" + ll.at(i), ll.at(i) + "/");
        }
      } else {
        copyIfNeeded(sPath + "/" + ll.at(i), sPathTo + "/" +  ll.at(i));
        sResourceFiles += QString("<file alias=\"" + sPrefix + ll.at(i) + "\">../Resources/") + sPrefix + ll.at(i) + "</file>\n";
      }
    }
  }
}

bool Parser::copyIfNeeded(QString s, QString s2, bool bContentOnly)
{
   bool bCopy = false;

   // copy always and change file date
   if (s2.endsWith("windows_application_icon.rc")){

     QFile f(s);
     if (!f.open(QIODevice::ReadOnly)){
       return false;
     } else {
       QByteArray ba = f.readAll();
       QFile f2(s2);
     
       if (!f2.open(QIODevice::Truncate | QIODevice::WriteOnly)){
         return false;
       } else {
         return f2.write(ba) == ba.size();
       }

     }
     return false;
   }

  if (QFile::exists(s2)){

    QFileInfo fi(s);
    QFileInfo fiCached(s2);

    bool b1 = false;
    bool b2 = false;
    bool b3 = false;

    if (bContentOnly){
      if (s.endsWith(".pro")){
        QString a = LoadFile(s);
        QString b = LoadFile(s2);

        QString sSearch = "#################";
        int n = a.lastIndexOf(sSearch);
        if (n >= 0) a = a.mid(n + sSearch.length());
        n = b.lastIndexOf(sSearch);
        if (n >= 0) b = b.mid(n + sSearch.length());
        
        bCopy = a != b;
      
      } else {
        bCopy = LoadFile(s) != LoadFile(s2);
      }

    } else {
      QString ss = fi.lastModified().toString();
      QString sss = fiCached.lastModified().toString();
      if ((b1 = (fi.lastModified() == fiCached.lastModified())) && (b2 = (fi.size() == fiCached.size())) && (bContentOnly && (b3 = (LoadFile(s) == LoadFile(s2))))){
      } else {
        bCopy = true;
    }
    }
  } else {
      bCopy = true;
  }

  if (bCopy){
   if (s.toLower() != s2.toLower()){
     // copy
//     Write_To_stdout("copyIfNeeded\n");
     QFile f(s2);
     f.remove();
     QFile::copy(s, s2);     
  //  Write_To_stdout("            copyIfNeeded\n");

   }
  }

  return bCopy;
} 

bool Parser::removeDirectory(QDir aDir, bool bFollowLinks)
{
    bool has_err = false;
    if (aDir.exists())
    {
      QFileInfoList entries = aDir.entryInfoList(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files);
      int count = entries.size();
      for (int idx = 0; ((idx < count) && (true != has_err)); idx++)
      {
        QFileInfo entryInfo = entries[idx];
        QString path = entryInfo.absoluteFilePath();
      //  QMessageBox::information(0, "RemoveDirectory", path+ "\n" + aDir.absolutePath());
        if (entryInfo.isDir())
        {
          has_err = removeDirectory(QDir(path), bFollowLinks);
         // QMessageBox::information(0, "1 err", path);
        }
        else if (entryInfo.isSymLink())
        {
          if (/*bFollowLinks && */QFileInfo(entryInfo.symLinkTarget()).isDir() && entryInfo.symLinkTarget().startsWith(aDir.absolutePath())){
            has_err = removeDirectory(QDir(path), bFollowLinks);
         //   QMessageBox::information(0, "1 err", path);
          }
          else {
            QFile file(path);
            if (!file.remove()){
              has_err = true;
//              QMessageBox::information(0, "2 err", path);
            }
          }
        }
          else
        {

          //if (path.endsWith("/plugins")) QMessageBox::information(0, "RemoveDirectory", "1"+path);

          QFile file(path);
          if (!file.remove()){
          //  QMessageBox::information(0, "QFile err", path);
            has_err = true;
          }
          //if (QFile::exists(path))QMessageBox::information(0, "RemoveDirectory", "1"+path);


        }
      }

      if (!aDir.rmdir(aDir.absolutePath())){
         // QMessageBox::information(0, "aDir err", aDir.absolutePath());

        has_err = true;
      }

      //QMessageBox::information(0, "RemoveDirectory", "2"+aDir.absolutePath());

    }
   // if (has_err){
     // QMessageBox::information(0, "___err", aDir.absolutePath ());
   // }

    return(has_err);
}

/*
QString Parser::plainString(QString s)
{
  #ifdef QTBASIC // OBJB
    if (s.length() >= 5) s = s.mid(1, s.length() - 2);
  #else // OBJB
    if (s.length() >= 5) s = s.mid(2, s.length() - 3);
  #endif // OBJB
  return s;
}*/

