﻿/*
'***************************************************************************
'Copyright (C) 2010-2011 KBasic Software: www.kbasic.com. All rights reserved.
'Source codes are created by Bernd Noetscher sales@kbasic.com 
'
'You may use Expression source codes under the terms of the GNU Public License (Version 3) as published 
'by the Free Software Foundation. 
'
'If you are interested in using the source codes for other licenses or 
'commercial development, you must buy commercial licenses from KBasic Software.
'***************************************************************************
*/


#include "Q7BCompiler.h"



bool Parser::parseFactor(_statements & Statements)
{  
  _expressions Expression;
  Expression.bReturn = false;
  bool bReturn = parseFactor(Expression);
  Statements = Expression.Statements;
  return bReturn;
}

bool Parser::parseFactor(_expressions & ReturnExpression)
{    

  bool bEnumParsing = !Me.sParsingPreviousValidEnumName.isEmpty();

  if (ReturnExpression.bReturn){    

#ifdef Q7BASIC
    if (parseTerm("True")){ ReturnExpression.Statements << Compiler::pushBoolean(true); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("False")){ ReturnExpression.Statements << Compiler::pushBoolean(false); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("Null")){ ReturnExpression.Statements << Compiler::pushNull(); ReturnExpression.sType = "id"; return true; }
#elif defined OBJB
    if (parseTerm("True")){ ReturnExpression.Statements << Compiler::pushBoolean(true); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("False")){ ReturnExpression.Statements << Compiler::pushBoolean(false); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("Yes")){ ReturnExpression.Statements << Compiler::pushBoolean(true); /*YES*/ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("No")){ ReturnExpression.Statements << Compiler::pushBoolean(false); /*NO*/ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("Nil")){ ReturnExpression.Statements << Compiler::pushNil(); ReturnExpression.sType = "id"; return true; }
    if (parseTerm("Null")){ ReturnExpression.Statements << Compiler::pushNull(); ReturnExpression.sType = "id"; return true; } //"[NSNull null]"
#elif defined DROIDBASIC
    if (parseTerm("True")){ ReturnExpression.Statements << Compiler::pushBoolean(true); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("False")){ ReturnExpression.Statements << Compiler::pushBoolean(false); ReturnExpression.sType = "Boolean"; return true; }
    if (parseTerm("Null")){ ReturnExpression.Statements << Compiler::pushNull(); ReturnExpression.sType = "id"; return true; }
#endif // Q7BASIC_OBJB
   
   // *** CONSTANTS
   
    if (parseBuiltinConstant(ReturnExpression)){ return true; }
  
    {
      QString sString;
      if (parseString(&sString)){ 
        ReturnExpression.Statements << Compiler::pushString(sString); 
        ReturnExpression.sType = "String"; 
        return true; 
      }
    }

    {
      QString sNumber;
      if (parseNumber(&sNumber)){ 
       
        if (sNumber.endsWith("D")){
          ReturnExpression.Statements << Compiler::pushDecimal(sNumber);
          ReturnExpression.sType = "Decimal";
        } else if (sNumber.startsWith("0x")){
          bool ok = false;
          ReturnExpression.Statements << Compiler::pushInteger(sNumber.toInt(&ok, 16));
          ReturnExpression.sType = "Integer";
        } else if (sNumber.contains(".") || sNumber.contains("e") || sNumber.contains("E")){
          ReturnExpression.Statements << Compiler::pushFloat(sNumber.toDouble());
          ReturnExpression.sType = "Float";
        } else { 
          ReturnExpression.Statements << Compiler::pushInteger(sNumber.toLongLong());
          ReturnExpression.sType = "Integer";
        }      
      
        return true; 
      }
    }

    

    if (bEnumParsing == false && parseTerm("IIf")){

      if (!parseTerm("(")){ error("'(' expected"); return false; }
      _expressions Condition;
      if (!parseCondition(Condition)){ error(""); return false; }
      if (!parseTerm(",")){ error("',' expected"); return false; }

      _expressions Expression;
      if (!parseExpression(Expression, true)){ error(""); return false; }
        
      if (!parseTerm(",")){ error("',' expected"); return false; }

      _expressions Expression2;
      if (!parseExpression(Expression2, true)){ error(""); return false; }        

      if (!parseTerm(")")){ error("')' expected"); return false; }

      ReturnExpression.Statements << Compiler::push("IIf", Condition.Statements, Expression.Statements, Expression2.Statements);
      ReturnExpression.sType = DEFAULTTYPE;
      return true;                           
    }

    if (parseTerm("Not")){ 
      _expressions Factor(ReturnExpression.sType, ReturnExpression.bReturn);
      if (!parseFactor(Factor)){ error(""); return false; }

      if (!typeCompatible(Factor.sType, "", "NOT")){ error("expression types incompatible or operator not allowed with the expression types"); return false; }

      ReturnExpression.Statements << Compiler::pushOperator("NOT", Factor.Statements);
      ReturnExpression.sType = Factor.sType;
      return true;
    }
    if (parseTerm("Flip")){ 
      _expressions Factor(ReturnExpression.sType, ReturnExpression.bReturn);
      if (!parseFactor(Factor)){ error(""); return false; }

      if (!typeCompatible(Factor.sType, "", "FLIP")){ error("expression types incompatible or operator not allowed with the expression types"); return false; }

      ReturnExpression.Statements << Compiler::pushOperator("FLIP", Factor.Statements);
      ReturnExpression.sType = Factor.sType;
      return true;
    }
    // *** 
    if (parseTerm("+")){ 
      _expressions Factor(ReturnExpression.sType, ReturnExpression.bReturn);
      if (!parseFactor(Factor)){ error(""); return false; }

      if (!typeCompatible(Factor.sType, "", "+")){ error("expression types incompatible or operator not allowed with the expression types"); return false; }

      ReturnExpression.Statements << Compiler::pushOperator("PLUS", Factor.Statements);
      ReturnExpression.sType = Factor.sType;
      return true;
    }  
    // *** 
    if (parseTerm("-")){ 
      _expressions Factor(ReturnExpression.sType, ReturnExpression.bReturn);
      if (!parseFactor(Factor)){ error(""); return false; }

      if (!typeCompatible(Factor.sType, "", "-")){ error("expression types incompatible or operator not allowed with the expression types"); return false; }

      ReturnExpression.Statements << Compiler::pushOperator("MINUS", Factor.Statements);
      ReturnExpression.sType = Factor.sType;
      return true;
    }

    // enum 
    if (parseEnum(ReturnExpression)){
      return true;
    }
  }

  if (parseTerm("(")){ 
    _expressions Factor(ReturnExpression.sType, ReturnExpression.bReturn);
    if (!parseExpression(Factor)){ error(""); return false; }
    if (!parseTerm(")")){ error(""/*) expected*/); return false; }

    ReturnExpression.Statements << Compiler::pushOperator("BRACES", Factor.Statements);
    ReturnExpression.sType = Factor.sType;
    return true;
  }    

  if (bEnumParsing){
    return false;
  }
  /*
    // type 
    if (parseType(ReturnExpression)){
      return true;
    }
    */

  if (Me.nLine == 280 && Me.sFileName == "CodeView.QWidget.q7b"){
    Me.nLine = Me.nLine;
  }
  if (parseBuiltinSubFunction(ReturnExpression)){ return true; }

  _variableconstant Variable;
  _statements VariableStatements;
  //_BEGIN_

  
  bool bFakeAlreadyHandled = false;
  if (parseVariableOrObjectIdentifierOrProcedureCall(VariableStatements, Variable, false, ReturnExpression.bReturn)){  
    QString sQObjectPropertyOrKVC;
    if (parseQObjectPropertyOrKVC(&sQObjectPropertyOrKVC)){ // was passiert wenn auf object ohne qobject basis zugegriffen wird z.b. qstring ?? wird in der runtime gehandelt
      ReturnExpression.Statements << Compiler::pushBuiltinSubFunction("PROPERTY", ReturnExpression.bReturn, VariableStatements, Compiler::pushExpression(Compiler::pushString(sQObjectPropertyOrKVC)));
      ReturnExpression.sType = DEFAULTTYPE;
      return true;
    } else if (parseBuiltinClassOrControlOrObject_PropertyOrProcedure(ReturnExpression, Variable, VariableStatements, &bFakeAlreadyHandled)){
      QString wert = Compiler::decode(ReturnExpression.Statements);
        return true;
    } else {
   
      ReturnExpression.Statements << VariableStatements;
      ReturnExpression.sType = Variable.sType;
      ReturnExpression.bAlloc = Variable.bAlloc;
      return true;
    }
  }

  return false;
}





