/*
    This file contains declarations of all actions, which can be triggered by user.
    All this actions implemented with using of undo framework.
*/

#ifndef COMMANDS_H
#define COMMANDS_H
#include <QUndoCommand>
#include <QPointer>
#include <QPoint>
#include <QVariantMap>
#include "widgetmanager.h"
#include <QStringList>
/*
    This command asks WidgetManager for deletion of selected widgets
*/
class DeleteCommand : public QUndoCommand
{
public :
    DeleteCommand ( WidgetManager &W , QUndoCommand *parent = 0 ) ;

    void undo () ;
    void redo () ;
private :
    WidgetManager * WM ;
    QList<QPointer<QWidget> > selList ;
} ;

/*
    This command asks WidgetManager for morphing of selected widgets to another class
*/
class MorphCommand : public QUndoCommand
{
public :
    MorphCommand ( WidgetManager &W , QString className , QUndoCommand *parent = 0 ) ;

    void undo () ;
    void redo () ;
private :
    WidgetManager * WM ;
    QList < QPointer < QWidget > > selList ;
    QString newClassName ;
    QStringList prevClassNames ;
    QList < QPointer < QWidget > > widgets ;
} ;

/*
    This command copies a widget and deletes previous instance by WidgetManager methods
*/
class MoveAndDelCommand : public QUndoCommand
{
public :
    MoveAndDelCommand ( WidgetManager & W , QWidget * par  , QPoint hs ,
               QUndoCommand * parent = 0 ) ;
    ~MoveAndDelCommand () ;
    void undo () ;
    void redo () ;
private :
    QList < QPointer < QWidget > > widgets ;
    QList < QPointer < QWidget > > oldWidgets ;

    QWidget * parentWidget ;
    QPoint hotSpot ;
    QString clbdText ;
    WidgetManager * WM ;
    QPoint offset ;
} ;


/*
    This command moves selected widgets.
*/
class MoveCommand : public QUndoCommand
{
public :
    MoveCommand ( WidgetManager &W , QPoint delta ,
               QUndoCommand * parent = 0 ) ;

    void undo () ;
    void redo () ;
    bool mergeWith ( const QUndoCommand * command ) ;
    int id  () const ;
    static void endMoving () { movingIdst ++ ; }
private :
    QList < QPointer< QWidget > > selList ;
    QPoint delta ;
    static quint64 movingIdst ;
    quint64 movingId ;
    WidgetManager * WM ;

} ;

/*
    This command resizes selected widgets.
*/
class ResizeCommand : public QUndoCommand
{
public :
    ResizeCommand ( WidgetManager &W , QPoint delta , int i ,
               QUndoCommand * parent = 0 ) ;
    void undo () ;
    void redo () ;
    bool mergeWith ( const QUndoCommand * command ) ;
    int id  () const ;
    static void endResizing () { resizingIdst ++ ; }
private:
    WidgetManager * WM ;
    QList < QPointer< QWidget > > selList ;
    QPoint delta ;
    static quint64 resizingIdst ;
    quint64 resizingId ;
    int direction ;
} ;

/*
    This command changes properties of selected widgets.
*/
class EditCommand : public QUndoCommand
{
public:
    EditCommand ( WidgetManager &W , QVariantMap , QUndoCommand * parent = 0 ) ;

    void undo () ;
    void redo () ;

private:
    QVariantMap props ;
    QList < QVariantMap > oldProps ;
    WidgetManager * WM ;
    QList < QPointer< QWidget > > selList ;
} ;

/*
    This command alignes widgets.
*/
class AlignCommand : public QUndoCommand
{
public:
    AlignCommand ( WidgetManager * W , WidgetManager::AligningSide , QUndoCommand * parent = 0 ) ;

    void undo () ;
    void redo () ;

private:
    QList < QPointer< QWidget > > selList ;
    WidgetManager * WM ;
    WidgetManager::AligningSide aSide ;
    QHash < QWidget * , QPoint > oldPos ;
} ;
#endif // COMMANDS_H
