/*
    This file containes declaraion of DesigningArea class. It inherits QFrame and
    provides features like drawing grid, accepting Drag&Drop, etc.
*/

#ifndef DESIGNINGAREA_H
#define DESIGNINGAREA_H
#include <QWidget>
#include <QFrame>
#include <QPointer>


class SelectionSquares ;
class DesigningArea : public QFrame
{
    Q_OBJECT
public :
    DesigningArea ( QWidget * parent = 0 ) ;
    // Returns current selection objects list
    QList < QPointer < SelectionSquares > > getSelections () ;

    // Sets grid step. If argument is 1, grid will be swithed off.
    void setGrid ( int delta = 10 ) ;
    // Rounds point to closest grid point
    QPoint pointToGrid ( QPoint p ) ;
    // Round each of corners to closest grid points
    QRect rectToGrid ( QRect r ) ;

    // Emits widthSmall
    void emitWidthSmall ( int d ) ;
    // Emits heightSmall
    void emitHeightSmall ( int d ) ;
    // Emits geomSnall
    void emitGeomSmall ( int h , int w ) ;
    // Sets drawing of focus-rect to true or false
    void drawFocus ( bool b ) ;

    // Defines moving step in pixels
    static const int arrowMovingDelta ;
    // Defines resizing step in pixels
    static const int arrowResizingDelta ;
    // Set the mode of tooltip with cursor position
    void setShowCursorPos ( bool b ) ;
    // returns true if cursor showing is enabled
    bool getShowCursorPos (  ) ;
    //returns grid step ;
    int getGridDelta() ;

    void setMode ( bool b ) ;

    QWidget * currentWidget () ;
signals :

    // Emitted when user wants to move selected widgets
    void movingWidgets (  QPoint delta ) ;
    // Emitted when user wants to resize selected widgets
    void resizeWidgets ( QPoint delta , int i ) ;
    // Emitted when moving actions sequence is finished
    void endMoving () ;
    // Emitted when resizing actions sequence is finished
    void endResizing () ;
    // Emitted when user interacts with this DesigningArea
    void inFocus ( DesigningArea * ) ;
    // Emitted when widgets starts to require greater width for full displaying
    //---MOVE---
    void widthSmall ( int d ) ;
    // Emitted when widgets starts to require greater height for full displaying
    //---MOVE---
    void heightSmall ( int d ) ;
    //---MOVE---
    void geomSmall ( int h , int w ) ;

    // Emitted when user drops widgets. Drag and drop is using.
    void moveWidgetByPaste ( QWidget * parent , QPoint hotspot ) ;
    // Emitted when user executes context menu (makes right-click).
    void contextMenuSignal ( QWidget * ) ;
    // Emitted when user drags widgets.
    void widgetsDragged ( QWidget * ) ;
    // Emitted when user selects widgets.
    void widgetsSelected ( QList < QPointer < QWidget > > list , bool withControl ) ;
    // Emitted when user clicks on free space
    void clearSelection () ;
    // Emitted when it is time to show tooltip for widget
    void widgetTooltip( QWidget * w ) ;

    //clicked
    void leftClicked () ;
protected :
    void dragEnterEvent ( QDragEnterEvent * event ) ;
    void dropEvent (QDropEvent * event ) ;
    void dragMoveEvent ( QDragMoveEvent * ) ;
    void mousePressEvent (QMouseEvent * event ) ;
    void mouseReleaseEvent ( QMouseEvent * event ) ;
    void mouseMoveEvent ( QMouseEvent * event ) ;
    void mouseDoubleClickEvent ( QMouseEvent * event ) ;
    bool eventFilter ( QObject * watched , QEvent * event ) ;
    void paintEvent (QPaintEvent * ) ;
    void keyPressEvent  ( QKeyEvent * event ) ;
    void keyReleaseEvent ( QKeyEvent * event ) ;
private :
    // Pointer to last clicked widget
    QWidget * movingWidget ;
    // Used for selection frame drawing
    QPoint prevMovingPos ;
    // Stores initial point of selection frame
    QPoint startSelectionPos ;
    // Stores current end point of selection frame. Changed in mouseMoveEvent
    QPoint endSelectionPos ;
    // Defines selection frame state
    bool drawingSelection ;
    // Defines if there is a widget under cursor while moving
    bool moving ;

    //QPoint curMousePos ;
    // Defines grid step
    int gridDelta ;
    // Stores mode of cursor pos showing
    bool showCursorPos ;

    bool mode ;
private slots :
} ;

#endif // DESIGNINGAREA_H
