/*
    This file containes decalrations of classes, which are used
    for selecting feature of WidgetManager
*/

#ifndef SELECTIONSQUARES_H
#define SELECTIONSQUARES_H

#include <QWidget>
#include <QPainter>
#include <QPointer>
#include <QMouseEvent>
#include <QRect>

/*
    This class represents one "square". Such squares are often used for marking
    some visible object as "selected"
*/
class SelectionSquare : public QWidget
{
    Q_OBJECT
    void mouseMoveEvent ( QMouseEvent * ) ;
    void mousePressEvent ( QMouseEvent * ) ;
    void mouseReleaseEvent ( QMouseEvent * ) ;
    void paintEvent ( QPaintEvent * ) ;
    qreal s ;
    QCursor cursor ;
    qreal xCoef ;
    qreal yCoef ;
    bool dragging ;
    QPoint oldPos ;
public :
    SelectionSquare ( qreal side , QCursor cursor , qreal xc , qreal yc , QWidget * parent = 0 ) ;
    void updatePos () ;
signals :
    //Emitted when user drags this square
    void moveSignal ( QPoint d ) ;
    void endMoving () ;
} ;

/*
    This class represent "selection". It is the set of SelectionSquare objects and info about target widget
*/
class SelectionSquares : public QWidget
{
    Q_OBJECT
    // Stores the pointer to selected widget
    QPointer < QWidget > target ;
    // Stores pointers to SelectionSquare objects
    QList< SelectionSquare * > squares ;
    // Size of square side
    qreal s ;
public :
    SelectionSquares ( QWidget * trg , QWidget * parent = 0 ) ;
    // Sets target to trg
    void attach ( QWidget * trg ) ;
    // Returns current target
    QWidget * getTarget () ;
protected :
public slots :
    // Changes geometry of this object and moves all SelectionSquare objects to corresponding places
    void tuneGeometry () ;
private slots :
    // handles movings of SelectionSquare objects
    void squareMove ( QPoint p ) ;
    void endMoveSlot () ;
signals :
    void moveSignal ( QPoint d , int i  ) ;
    void endMoveSignal () ;
} ;

#endif // SELECTIONSQUARES_H
