/*
    This file containes declarations of "factory" class and helper classes.
    They are used for creating palettes and for creating new widget by its class name.
*/

#ifndef WIDGETFACTORY_H
#define WIDGETFACTORY_H

#include <QObject>
#include <QWidget>
#include <QRadioButton>
#include <QCheckBox>
#include <QPushButton>
#include <QFrame>
#include <QLabel>
#include <QLineEdit>
#include <QTabWidget>
#include <QHash>
#include <QPointer>
#include <QVariant>
#include <QSet>
#include <QAction>
#include "constants.h"
#include "relativelayout.h"
#include <QDebug>



#include "Q7B_Controls.h"

#include <QComboBox>
#include <QGroupBox>
#include <QListWidget>
#include <QProgressBar>
#include <QToolButton>
#include <QCommandLinkButton>
#include <QDialogButtonBox>
#include <QListView>
#include <QTreeView>
#include <QTableView>
#include <QColumnView>
#include <QStandardItemModel>
#include <QListWidget>
#include <QTreeWidget>
#include <QTableWidget>
#include <QScrollArea>
#include <QToolBox>
#include <QStackedWidget>
#include <QMdiArea>
#include <QDockWidget>
#include <QFontComboBox>
#include <QTextEdit>
#include <QPlainTextEdit>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QTimeEdit>
#include <QDateEdit>
#include <QDateTimeEdit>
#include <QDial>
#include <QScrollBar>
#include <QSlider>
#include <QTextBrowser>
#include <QGraphicsView>
#include <QCalendarWidget>
#include <QLCDNumber>

/*
    Abstract class which is used by WidgetFactory for storing pointers to
    inherited classes.
*/
class AbstractCreator : public QObject
{
    Q_OBJECT
    int count ;
protected:
    QHash < QString , QPointer < QAction > > actions ;
    QList < QPointer < QObject > > allInst ;
    virtual bool addChild ( QPointer < QWidget > parent , QPointer < QWidget > w ) = 0 ;
    virtual QPointer< QWidget > removeChild (  QPointer < QWidget >  , QPointer < QWidget >  w  ) { return w ; }
    virtual QWidget * create() = 0;
    virtual QWidget * createP() = 0;
    virtual void assignProperties ( QPointer < QWidget > , QVariantMap ){}
    virtual void copyProperties( QPointer < QWidget > , QVariantMap &, bool  ) {}
    void basicAssignProps( QPointer < QWidget > , QVariantMap pr);
    void basicCopyProperties( QPointer < QWidget >  , QVariantMap & props , bool deleteId ) ;
public :
    AbstractCreator () { count = 0 ; }
    // Returns name of class, which objects are created by AbstractCreator inherited class
    virtual  QString className () = 0 ;
    // Creates new widget and returns pointer to it.
    QWidget * createWidget (  )
    {
        QWidget * w = create () ;
        allInst.append ( w ) ;
        return w ;
    }
    QWidget * createForPalette ()
    {
        return createP () ;
    }
    // This function must add a child to a widget, if it is a container
    bool addChildToWidget ( QPointer < QWidget > parent , QPointer < QWidget > w )
    {
        if ( w )
        {
            bool b = addChild( parent , w ) ;
            if ( b && ! allInst.contains ( w.data() ) )
                allInst.append( w.data() ) ;
            return b ;
        }
        else
            return false ;
    }

    virtual void addEventFilterToWidget (  QPointer < QWidget > parent , QPointer < QWidget > w )
    {
        w->installEventFilter ( parent ) ;
    }
    // Must return list of children of this widget if it is a container
    virtual QList< QPointer< QWidget > > getChildren ( QPointer < QWidget > parent )
    {
        QList< QPointer< QWidget > > list ;
        foreach ( QObject * w , parent->children () )
        {
            if ( w->property ( SEL_SQUARES_MARK ).toBool () == true )
                continue ;
            else
            {
                QWidget * widget = qobject_cast< QWidget * > ( w ) ;
                if ( widget )
                    list.append ( widget ) ;
            }
        }
        return list ;
    }
    // Must remove a child from a widget
    QPointer< QWidget > removeChildFromWidget (  QPointer < QWidget > p , QPointer < QWidget >  w  )
    {
        allInst.removeAll ( w.data() ) ;
        removeChild ( p , w ) ;
        return w ;
    }

    //int getCount () { return count ; }

    virtual QList < QPointer < QAction > > getActions ( QPointer < QWidget > )
    {
        return QList < QPointer < QAction > > () ;
    }
    void assignPropertiesToWidget ( QPointer < QWidget > , QVariantMap pr);
    void copyPropertiesFromWidget( QPointer < QWidget > , QVariantMap &props, bool deleteId ) ;

    QString getUniqueName(QString s)
    {
        QStringList allNames ;
        foreach ( QObject * o , allInst )
        {
            if (o)
                allNames.append( o->objectName() ) ;
            else
                allInst.removeAll( o ) ;
        }
        if ( s != "" && ! allNames.contains(s) )
            return s ;
        QString newName;
        QString templ = className() + "%1";
        int t = 1;

        while ( allNames.contains( newName = templ.arg(t ++) ) );
        return newName;
    }
} ;

void updateRecursive ( QWidget * w ) ;


class CommandButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  {return false ; }
    QWidget * create ( )
    {
        CommandButton * b = new CommandButton ( ) ;
        b->setToolTip("CommandButton");
        b->setText("CommandButton");
        b->setGeometry( 0 , 0 , 100 , 20 ) ;
        return b ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "CommandButton" ; }


} ;

class TreeViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  {return false ; }
    QWidget * create ( )
    {
        TreeView * b = new TreeView ( ) ;
        b->setToolTip("TreeView");
        b->setGeometry( 0 , 0 , 100 , 20 ) ;
        return b ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "TreeView" ; }


} ;

class ListViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  {return false ; }
    QWidget * create ( )
    {
        ListView * b = new ListView ( ) ;
        b->setToolTip("ListView");
        b->setGeometry( 0 , 0 , 100 , 20 ) ;
        return b ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ListView" ; }


} ;

class QPushButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  {return false ; }
    QWidget * create ( )
    {
        QPushButton * b = new QPushButton ( "QPushButton" ) ;
        b->setToolTip("QPushButton");
        b->setGeometry( 0 , 0 , 100 , 20 ) ;
        return b ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QPushButton" ; }


} ;

class QCheckBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create (  )
    {
        QCheckBox * n = new QCheckBox ( "QCheckBox" ) ;
        n->setToolTip("QCheckBox");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QCheckBox" ; }


} ;

class CheckBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create (  )
    {
        CheckBox * n = new CheckBox ( "CheckBox" ) ;
        n->setToolTip("CheckBox");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "CheckBox" ; }


} ;

class QLineEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QLineEdit * n = new QLineEdit ( "QLineEdit" ) ;
        n->setToolTip("QLineEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QLineEdit" ; }


} ;

class QTextEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTextEdit * n = new QTextEdit ( "QTextEdit" ) ;
        n->setToolTip("QTextEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTextEdit" ; }


} ;

class QPlainTextEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QPlainTextEdit * n = new QPlainTextEdit ( "QPlainTextEdit" ) ;
        n->setToolTip("QPlainTextEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QPlainTextEdit" ; }


} ;

class QSpinBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QSpinBox * n = new QSpinBox ( ) ;
        n->setToolTip("QSpinBox");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QSpinBox" ; }


} ;

class QDialCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QDial * n = new QDial ( ) ;
        n->setToolTip("QDial");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDial" ; }


} ;

class QScrollBarHorizontalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QScrollBar * n = new QScrollBar ( ) ;
        n->setToolTip("QScrollBar");
        n->setOrientation(Qt::Horizontal);
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QScrollBar" ; }


} ;

class QScrollBarVerticalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QScrollBar * n = new QScrollBar ( ) ;
        n->setToolTip("QScrollBar");
        n->setOrientation(Qt::Vertical);
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QScrollBar" ; }


} ;

class QSliderHorizontalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QSlider * n = new QSlider ( ) ;
        n->setToolTip("QSlider");
        n->setOrientation(Qt::Horizontal);
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QSlider" ; }


} ;

class QSliderVerticalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QSlider * n = new QSlider ( ) ;
        n->setToolTip("QSlider");
        n->setOrientation(Qt::Vertical);
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QSlider" ; }


} ;

class QCalendarWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QCalendarWidget * n = new QCalendarWidget ( ) ;
        n->setToolTip("QCalendarWidget");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QCalendarWidget" ; }


} ;

class QLCDNumberCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QLCDNumber * n = new QLCDNumber ( ) ;
        n->setToolTip("QLCDNumber");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QLCDNumber" ; }


} ;


class QProgressBarCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QProgressBar * n = new QProgressBar ( ) ;
        n->setToolTip("QProgressBar");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QProgressBar" ; }


} ;


class QTextBrowserCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTextBrowser * n = new QTextBrowser (  ) ;
        n->setToolTip("QTextBrowser");
        n->setText("QTextBrowser");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTextBrowser" ; }


} ;

class QGraphicsViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QGraphicsView * n = new QGraphicsView (  ) ;
        n->setToolTip("QGraphicsView");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QGraphicsView" ; }


} ;

class QDoubleSpinBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QDoubleSpinBox * n = new QDoubleSpinBox ( ) ;
        n->setToolTip("QDoubleSpinBox");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDoubleSpinBox" ; }


} ;

class QTimeEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTimeEdit * n = new QTimeEdit ( ) ;
        n->setToolTip("QTimeEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTimeEdit" ; }


} ;

class QDateEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QDateEdit * n = new QDateEdit ( ) ;
        n->setToolTip("QDateEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDateEdit" ; }


} ;

class QDateTimeEditCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QDateTimeEdit * n = new QDateTimeEdit ( ) ;
        n->setToolTip("QDateTimeEdit");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDateTimeEdit" ; }


} ;

class QToolButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QToolButton * n = new QToolButton (  ) ;
        n->setToolTip("QToolButton");
        n->setText("QToolButton");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QToolButton" ; }


} ;

class QCommandLinkButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QCommandLinkButton * n = new QCommandLinkButton (  ) ;
        n->setToolTip("QCommandLinkButton");
        n->setText("QCommandLinkButton");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QCommandLinkButton" ; }


} ;

class TextBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        TextBox * n = new TextBox ( "TextBox" ) ;
        n->setToolTip("TextBox");
        n->setGeometry ( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "TextBox" ; }


} ;

class QDialogButtonBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QDialogButtonBox * n = new QDialogButtonBox (  ) ;
        n->setToolTip("QDialogButtonBox");
        n->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDialogButtonBox" ; }

} ;

class QListViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QListView * n = new QListView (  ) ;
        n->setToolTip("QListView");
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        QStandardItemModel *model = new QStandardItemModel(1, 1);
        model->setData( model->index( 0, 0 ), "QListView", Qt::DisplayRole );
        n->setModel(model);

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QListView" ; }

} ;

class QTreeViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTreeView * n = new QTreeView (  ) ;
        n->setToolTip("QTreetView");
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        QStandardItemModel *model = new QStandardItemModel(1, 1);
        model->setData( model->index( 0, 0 ), "QTreetView", Qt::DisplayRole );
        n->setModel(model);

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTreeView" ; }

} ;

class QTableViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTableView * n = new QTableView (  ) ;
        n->setToolTip("QTableView");
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        QStandardItemModel *model = new QStandardItemModel(1, 1);
        model->setData( model->index( 0, 0 ), "QTableView", Qt::DisplayRole );
        n->setModel(model);

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTableView" ; }

} ;

class QColumnViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QColumnView * n = new QColumnView (  ) ;
        n->setToolTip("QColumnView");
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        QStandardItemModel *model = new QStandardItemModel(1, 1);
        model->setData( model->index( 0, 0 ), "QColumnView", Qt::DisplayRole );
        n->setModel(model);

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QColumnView" ; }

} ;

class QListWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QListWidget * n = new QListWidget (  ) ;
        n->setToolTip("QListWidget");
        n->addItem("QListWidget");
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

   //     return n ;

        QPixmap pixmap (n->size ());
        n->render (&pixmap);

        QLabel *l = new QLabel();
        l->setPixmap(pixmap);


        return l ;

    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QListWidget" ; }

} ;

class QTreeWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTreeWidget * n = new QTreeWidget (  ) ;
        n->setToolTip("QTreeWidget");
        n->setColumnCount(1);
        n->insertTopLevelItem(0, new QTreeWidgetItem(QStringList("QTreeWidget")));
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTreeWidget" ; }

} ;

class QTableWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QTableWidget * n = new QTableWidget (  ) ;
        n->setToolTip("QTableWidget");
        n->setRowCount(1);
        n->setColumnCount(1);
        n->setItem(0, 0, new QTableWidgetItem("QTableWidget"));
       // n->setColumnCount(1);
       // n->insertTopLevelItem(0, new QTableWidgetItem(QStringList("QTableWidget")));
        n->setMaximumHeight(40);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QTableWidget" ; }

} ;

class QComboBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QComboBox * n = new QComboBox (  ) ;
        n->setToolTip("QComboBox");
        n->addItem("QComboBox");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QComboBox" ; }

} ;

class QFontComboBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QFontComboBox * n = new QFontComboBox (  ) ;
        n->setToolTip("QFontComboBox");
        //n->addItem("QFontComboBox");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QFontComboBox" ; }

} ;

class ComboBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ComboBox * n = new ComboBox (  ) ;
        n->setToolTip("ComboBox");
        n->addItem("ComboBox");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ComboBox" ; }

} ;

class ComboBoxEditableCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ComboBox * n = new ComboBox (  ) ;
        n->setToolTip("ComboBox (Editable)");
        n->addItem("ComboBox (Editable)");
        n->setEditable(true);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ComboBox" ; }

} ;

class ListBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ListBox * n = new ListBox (  ) ;
        n->setToolTip("ListBox");
        n->addItem("ListBox");
        n->setMaximumHeight(40);
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ListBox" ; }

} ;

class ListBoxIconModeCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ListBox * n = new ListBox (  ) ;
        n->setToolTip("ListBox (IconMode)");
        n->addItem("ListBox (IconMode)");
        n->setViewMode(QListWidget::IconMode);
        n->setMaximumHeight(40);

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ListBox" ; }

} ;

class ImageBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ImageBox * n = new ImageBox (  ) ;
        n->setToolTip("ImageBox");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ImageBox" ; }

} ;

class TimerCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        Timer * n = new Timer (  ) ;
        n->setToolTip("Timer");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;

        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Timer" ; }

} ;

class ProgressBarCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ProgressBar * n = new ProgressBar (  ) ;
        n->setToolTip("ProgressBar");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ProgressBar" ; }

} ;

class ProgressBarSpinningCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        ProgressBar * n = new ProgressBar (  ) ;
        n->setToolTip("ProgressBar (Spinning)");
        n->setMinimum(0);
        n->setMaximum(0);
        n->setValue(0);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ProgressBar" ; }

} ;

class QRadioButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QRadioButton * n = new QRadioButton ( "QRadioButton" ) ;
        n->setToolTip("QRadioButton");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QRadioButton" ; }

} ;

class RadioButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >   , QPointer < QWidget >   )  { return false ; }
    QWidget * create ( )
    {
        QRadioButton * n = new QRadioButton ( "RadioButton" ) ;
        n->setToolTip("RadioButton");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
       // return n ;

        QPixmap pixmap (n->size ());
        n->render (&pixmap);

        QLabel *l = new QLabel();
        l->setPixmap(pixmap);


        return l ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "RadioButton" ; }

} ;

class QWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        QWidget * n = new QWidget ( ) ;
        n->setToolTip("QWidget");
        n->setMinimumSize(100, 20);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QWidget" ; }


} ;

class QMdiAreaCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        QMdiArea * n = new QMdiArea (  ) ;
        n->setToolTip("QMdiArea");
        n->setMaximumSize(100, 20);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QMdiArea" ; }


} ;

class QDockWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        QDockWidget * n = new QDockWidget ( "QDockWidget" ) ;
        n->setToolTip("QDockWidget");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        n->setMaximumSize(100, 20);
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QDockWidget" ; }


} ;

class QLabelCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        QLabel * n = new QLabel ( "QLabel" ) ;
        n->setToolTip("QLabel");        
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QLabel" ; }


} ;

class LabelCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        Label * n = new Label ( "Label" ) ;
        n->setToolTip("Label");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Label" ; }


} ;

class ImageButtonCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        ImageButton * n = new ImageButton ( "ImageButton" ) ;
        n->setToolTip("ImageButton");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "ImageButton" ; }


} ;

class BoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        Box * n = new Box ( ) ;
        n->setToolTip("Box");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Box" ; }


} ;

class TextBoxPasswordCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        TextBox * n = new TextBox ("Password (TextBox)") ;
        n->setToolTip("Password (TextBox)");
        n->setEchoMode(QLineEdit::Password);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "TextBox" ; }


} ;

class LabelLinkCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        TextBox * n = new TextBox ("Link (Label)") ;
        n->setToolTip("Link (Label)");
        n->setEchoMode(QLineEdit::Password);
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Label" ; }


} ;
/*
class LineHorizontalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        Line * n = new Line ("Horizontal Line") ;
        n->setToolTip("Horizontal Line)");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Line" ; }


} ;*/

class FormViewCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        FormView * n = new FormView () ;
        n->setToolTip("FormView)");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "FormView" ; }


} ;

/*
class LineVerticalCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        Line * n = new Line ("Vertical Line") ;
        n->setToolTip("Vertical Line)");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "Line" ; }


} ;*/

class QGroupBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >  parent , QPointer < QWidget >  w )  { w->setParent ( parent ) ; return true ; }
    QWidget * create  (  )
    {
        QGroupBox * r = new QGroupBox (  ) ;
        r->setToolTip("QGroupBox");
        r->setTitle("QGroupBox");
        r->setMinimumSize ( 20 , 20 ) ;
        r->setGeometry( 0 , 0 , 100 , 100 ) ;
        r->setAcceptDrops ( true ) ;
        return r ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QGroupBox" ; }


} ;

class QFrameCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >  parent , QPointer < QWidget >  w )  { w->setParent ( parent ) ; return true ; }
    QWidget * create  (  )
    {
        QFrame * r = new QFrame (  ) ;
        r->setToolTip("QFrame");
        r->setMinimumSize ( 20 , 20 ) ;
        r->setFrameShape ( QFrame::Panel ) ;
        r->setGeometry( 0 , 0 , 100 , 100 ) ;
        r->setAcceptDrops ( true ) ;
        return r ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QFrame" ; }


} ;

class QScrollAreaCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget > , QPointer < QWidget >  )  { return false ; }
    QWidget * create ( )
    {
        QScrollArea * n = new QScrollArea ( ) ;
       // n->setText("QScrollArea");
        n->setToolTip("QScrollArea");
        n->setGeometry( 0 , 0 , 100 , 20 ) ;
        return n ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QScrollArea" ; }


} ;

/*
class QScrollAreaCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >  parent , QPointer < QWidget >  w )  {w->setParent ( parent ) ; return true ; }
    QWidget * create  (  )
    {
        QScrollArea * r = new QScrollArea (  ) ;
        r->setToolTip("QScrollArea");
        r->setGeometry( 0 , 0 , 100 , 100 ) ;
        r->setAcceptDrops ( true ) ;
        return r ;
    }
public :
    QString className () { return "QScrollArea" ; }


} ;*/

class QToolBoxCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >  parent , QPointer < QWidget >  w )  {w->setParent ( parent ) ; return true ; }
    QWidget * create  (  )
    {
        QToolBox * r = new QToolBox (  ) ;
        r->setToolTip("QToolBox");
        r->addItem ( new QWidget( ), "QToolBox" );
        r->setGeometry( 0 , 0 , 100 , 100 ) ;
        r->setAcceptDrops ( true ) ;
        return r ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QToolBox" ; }


} ;

class QStackedWidgetCreator : public AbstractCreator
{
    bool addChild ( QPointer < QWidget >  parent , QPointer < QWidget >  w )  {w->setParent ( parent ) ; return true ; }
    QWidget * create  (  )
    {
        QStackedWidget * r = new QStackedWidget (  ) ;
        r->setToolTip("QStackedWidget");
        r->addWidget ( new QLabel( "QStackedWidget" ) );
        r->setGeometry( 0 , 0 , 100 , 100 ) ;
        r->setAcceptDrops ( true ) ;
        return r ;
    }

    QWidget * createP ()
    {
       return create() ;
    }

public :
    QString className () { return "QStackedWidget" ; }


} ;


class QTabWidgetCreator : public AbstractCreator
{
    Q_OBJECT
    QPointer < QTabWidget > currentWidget ;
    bool addChild (  QPointer < QWidget >  parent , QPointer < QWidget >  w ) ;
    QPointer< QWidget > removeChild (  QPointer < QWidget > parent , QPointer < QWidget >  w  )
    {
        QTabWidget * r = qobject_cast< QTabWidget * > ( parent ) ;
        if ( r )
        {
            if ( r->indexOf ( w ) != - 1 )
                r->removeTab ( r->indexOf ( w ) ) ;
            return w ;
        }
        else
        {
            return w ;
        }
    }

    QWidget * create ( )
    {
        QTabWidget * r = new QTabWidget (  ) ;
        r->setMinimumSize ( 20 , 20 ) ;
        r->setGeometry( 0 , 0 , 150 , 150 ) ;
        r->setAcceptDrops ( true ) ;
        foreach ( QObject * ob , r->children () )
        {
            QWidget * w = qobject_cast < QWidget * > ( ob ) ;
            if ( w )
            {
                w->setAcceptDrops ( true ) ;
                w->installEventFilter ( r ) ;
            }
        }

        //w->deleteLater();
        return r ;
    }
    QWidget * createP () ;

public :
    QTabWidgetCreator () : AbstractCreator ()
    {
        actions.insert ( "Add tab" , new QAction ( "Add tab" , 0 ) ) ;
        actions.insert ( "Remove tab", new QAction ( "Remove tab" , 0 ) ) ;

        connect ( actions [ "Add tab" ].data() , SIGNAL(triggered()) , SLOT(newTabSlot())) ;
        connect ( actions [ "Remove tab" ].data() , SIGNAL(triggered()) , SLOT(removeTabSlot())) ;
    }
    QString className () { return "QTabWidget" ; }
    void assignProperties (QPointer < QWidget >  w, QVariantMap pr);
    void copyProperties( QPointer < QWidget > w , QVariantMap &props, bool  )
    {
        QTabWidget * r = qobject_cast< QTabWidget * > ( w ) ;
        QVariantMap l ;
        QStringList to ;
        for (int i = 0 ; i < r->count() ; i ++ )
        {
            l.insert ( r->widget( i )->objectName() , r->tabText ( i ) ) ;
            to.append ( r->widget( i )->objectName() ) ;
        }
        props.insert( "tabs" , l ) ;
        props.insert( "tabs_order" , to ) ;
    }

    void addEventFilterToWidget ( QPointer < QWidget > parent , QPointer < QWidget > w )
    {
        w->installEventFilter ( parent ) ;
    }

    QList< QPointer< QWidget > > getChildren ( QPointer < QWidget >  parent )
    {
        QTabWidget * r = qobject_cast< QTabWidget * > ( parent ) ;
        if ( r )
        {
            QList< QPointer< QWidget > > list ;
            for ( int i = 0 ; i < r->count () ; i ++ )
            {
                list.append(r->widget ( i ));
                //qDebug() << list.last()->objectName() << list.last()->metaObject()->className();
            }
            return list ;
        }
        else
        {
            QList< QPointer< QWidget > > list ;
            foreach ( QObject * w , parent->children () )
            {
                if ( w->property ( SEL_SQUARES_MARK ).toBool () == true )
                    continue ;
                else
                {
                    QWidget * widget = qobject_cast< QWidget * > ( w ) ;
                    if ( widget )
                        list.append ( widget ) ;
                }
            }
            return list ;
        }
    }

    virtual QList < QPointer < QAction > > getActions ( QPointer < QWidget > w ) ;
public slots:
    void newTabSlot () ;
    void removeTabSlot () ;
} ;

/*
    Provides factory for creating widgets for DesigningArea.
*/

class WidgetFactory : public QObject
{
    Q_OBJECT
    static WidgetFactory * instancePtr ;
    explicit WidgetFactory ( QObject * parent = 0 ) ;
    QList < AbstractCreator * > creators ;
    QHash < QString , AbstractCreator * > creatorsHash ;
    QMap < QString , QStringList > categories ;
    QSet < QString > noLayoutClassNames ;
public :
    // Creates new widget by given class name and returns pointer to it.
    QWidget * newWidgetByClassName ( QString cn, QString name, bool uName  ) ;
    QWidget * newWidgetByClassNameP ( QString cn , QString name ) ;
    // Creates full set of widgets
    QList < QWidget * > newAll () ;
    QList < QWidget * > newByCategory ( QString c ) ;
    QStringList getClassNames () ;
    QStringList getCategories () ;
    QStringList getClassNamesByCat ( QString c ) ;
    static WidgetFactory * instance () ;
    bool isClassSupported ( QString s ) ;
    bool addChildToWidget ( QWidget * parent , QWidget * child ) ;
    void addEventFilterToWidget ( QWidget * parent , QWidget * child ) ;
    QList< QPointer< QWidget > >  getChildren ( QWidget * parent ) ;
    QWidget * findKnownParent( QWidget * child ) ;
    QPointer< QWidget > removeChildFromWidget (  QWidget * parent, QWidget * w  ) ;
    //void copyToCB ( QString className ) ;
    QList < QPointer < QAction > > getSpecialActions ( QWidget * w ) ;
    void assignProperties (QWidget * w, QVariantMap pr);
    void copyProperties(QWidget * w, QVariantMap &props, bool deleteId ) ;
    void copyToCB( QWidget * w ) ;
    QVariantMap createChildPropList ( QWidget * w , QPoint posDelta , bool deleteId  ) ;
    QList<QPointer<QWidget> > restoreWidgets ( QWidget * parent , QVariantMap data , QWidget * mainFilter , bool uName ) ;
    void emitPasteNeeded(){emit pasteNeeded();}
    void emitDeleteNeeded( QList < QPointer <QWidget > > l ) { emit deleteNeeded ( l ) ; }

signals :
    void widgetsChanged() ;
    void pasteNeeded () ;
    void deleteNeeded ( QList < QPointer <QWidget > > ) ;
public slots :
} ;

#endif // WIDGETFACTORY_H
